define('app', function (require, _exports, module) {
  Promise.onUncaught = function (reason) {
    opera.postError(reason.message + '\n' + reason.stack);
  };

  var scriptUtils = require('utils/script');
  var VMDB = require('vmdb');
  var badges = require('badges');
  var requests = require('requests');
  var tabs = require('utils/tabs');
  var cache = require('utils/cache');
  _.i18n = require('utils/i18n');
  window._require = require;

  var vmdb = new VMDB;
  var app = module.exports = {version: '1.7.3'};

  var autoUpdate = function () {
    function check() {
      checking = true;
      return new Promise(function (resolve, reject) {
        if (!_.options.get('autoUpdate')) return reject();
        if (Date.now() - _.options.get('lastUpdate') >= 864e5)
          resolve(commands.CheckUpdateAll());
      }).then(function () {
        setTimeout(check, 36e5);
      }, function () {
        checking = false;
      });
    }
    var checking;
    return function () {
      checking || check();
    };
  }();

  function initMessenger() {
    var callbacks = [];
    return {
      connect: function (window, callback) {
        callbacks.push([window, callback]);
      },
      post: function (data) {
        data = JSON.parse(JSON.stringify(data));
        callbacks = callbacks.filter(function (item) {
          if (item[0].closed) return false;
          try {
            item[1](data);
          } catch (e) {
            return false;
          }
          return true;
        });
      },
    };
  }
  _.messenger = initMessenger();
  _.popupMessenger = initMessenger();

  var commands = app.commands = {
    NewScript: function () {
      return scriptUtils.newScript();
    },
    RemoveScript: function (id) {
      return vmdb.removeScript(id);
    },
    GetData: function () {
      return vmdb.getData().then(function (data) {
        data.options = _.options.getAll();
        data.version = app.version;
        return data;
      });
    },
    GetInjected: function (url) {
      var data = {
        isApplied: _.options.get('isApplied'),
        injectMode: _.options.get('injectMode'),
        version: app.version,
      };
      return data.isApplied
      ? vmdb.getScriptsByURL(url).then(function (res) {
        return _.assign(data, res);
      }) : data;
    },
    UpdateScriptInfo: function (data) {
      return vmdb.updateScriptInfo(data.id, data).then(function (script) {
        _.messenger.post({
          cmd: 'update',
          data: script,
        });
      });
    },
    SetValue: function (data) {
      return vmdb.setValue(data.uri, data.values);
    },
    SetOption: function (data) {
      _.options.set(data.key, data.value);
    },
    ExportZip: function (data) {
      return vmdb.getExportData(data.ids, data.values);
    },
    GetScript: function (id) {
      return vmdb.getScriptData(id);
    },
    GetMetas: function (ids) {
      return vmdb.getScriptInfos(ids);
    },
    Move: function (data) {
      return vmdb.moveScript(data.id, data.offset);
    },
    Vacuum: function () {
      return vmdb.vacuum();
    },
    ParseScript: function (data) {
      return vmdb.parseScript(data).then(function (res) {
        // var meta = res.data.meta;
        _.messenger.post(res);
        return res.data;
      });
    },
    CheckUpdate: function (id) {
      vmdb.getScript(id).then(vmdb.checkUpdate);
    },
    CheckUpdateAll: function () {
      _.options.set('lastUpdate', Date.now());
      vmdb.getScriptsByIndex('update', '"update"=1').then(function (scripts) {
        return Promise.all(scripts.map(vmdb.checkUpdate));
      });
    },
    ParseMeta: function (code) {
      return scriptUtils.parseMeta(code);
    },
    AutoUpdate: autoUpdate,
    GetPopup: badges.get,
    SetPopup: badges.set,
    InstallScript: function (data) {
      var params = encodeURIComponent(data.url);
      if (data.from) params += '/' + encodeURIComponent(data.from);
      if (data.text) cache.set(data.url, data.text);
      tabs.create('/options.html#confirm/' + params);
    },
    GetRequestId: function () {
      return requests.getRequestId();
    },
    HttpRequest: function (details, src) {
      requests.httpRequest(details, function (res) {
        src.postMessage({
          cmd: 'HttpRequested',
          data: res,
        });
      });
      return false;
    },
    AbortRequest: function (id) {
      return requests.abortRequest(id);
    },
  };

  if (+opera.version() < 12) {
    tabs.create('https://github.com/violentmonkey/violentmonkey-oex/wiki/Obsolete');
  } else {
    vmdb.initialized.then(function () {
      opera.extension.onmessage = function (e) {
        /*
        * o={
        * 	cmd: String,
        * 	src: {
        * 		id: String,
        * 		url: String,
        * 	},
        * 	callback: String,
        * 	data: Object
        * }
        */
        function finish(data, error) {
          e.source.postMessage({
            cmd: 'Callback',
            cmdFor: req.cmd,
            data: data,
            error: error,
          });
        }
        var req = e.data;
        var func = commands[req.cmd];
        if (func) {
          var res = func(req.data, e.source);
          if (res) return Promise.resolve(res)
          .then(function (data) {
            finish(data);
          }, function (err) {
            if (err) opera.postError(err.message, err.stack);
            finish(null, err);
          });
        }
        finish();
      };
      badges.init();
      setTimeout(autoUpdate, 2e4);
      opera.extension.tabs.onfocus = badges.get;
    });
  }
});

define('badges', function (require, _exports, module) {
  var tabs = require('utils/tabs');

  function cancel() {
    if (timer) {
      clearTimeout(timer);
      timer = null;
    }
  }
  function clear() {
    cancel();
    badges.button.badge.display = 'none';
    tabData = null;
  }
  function clearLater() {
    cancel();
    timer = setTimeout(clear, 200);
  }
  function getBadges() {
    var tab = tabs.get();
    try {
      tab.postMessage({cmd: 'GetPopup'});
      clearLater();
    } catch (e) {
      clear();
    }
  }
  function setBadges(data) {
    tabData = data;
    if (data.badge /*&& _.options.get('showBadge')*/) {
      badges.button.badge.textContent = data.badge > 99 ? '99+' : data.badge;
      badges.button.badge.display = 'block';
      cancel();
    } else {
      clear();
    }
    _.popupMessenger.post(data);
  }
  function updateIcon() {
    var isApplied = _.options.get('isApplied');
    badges.button.icon = '/images/icon18' + (isApplied ? '' : 'w') + '.png';
  }
  function show(show) {
    var toolbar = opera.contexts.toolbar;
    if (show) toolbar.addItem(badges.button);
    else toolbar.removeItem(badges.button);
  }
  function init() {
    badges.button = opera.contexts.toolbar.createItem({
      title: _.i18n('extName'),
      popup: {
        href: '/popup/index.html',
        width: 222,
        height: 100,
      },
      badge: {
        backgroundColor: '#808',
        color: 'white',
        display: 'none',
      },
    });
    updateIcon();
    show(_.options.get('showButton'));
  }
  var timer, tabData;
  var badges = module.exports = {
    init: init,
    get: getBadges,
    set: setBadges,
    update: updateIcon,
    getData: function () {
      return tabData;
    },
  };
});

define('vmdb', function (require, _exports, module) {
  var scriptUtils = require('utils/script');
  var tester = require('utils/tester');
  var tabsUtils = require('utils/tabs');

  function VMDB() {
    var _this = this;
    _this.initialized = _this.openDB().then(_this.initPosition.bind(_this));
    _this.checkUpdate = _this.checkUpdate.bind(_this);
  }

  /* ===============Data format 0.5==================
   * Database: Violentmonkey
   * scripts {
   * 		id: Auto
   * 		uri: String
   * 		custom: List-Dict	// Custom meta data
   * 		meta: List-Dict
   * 		enabled: 0|1
   * 		update: 0|1
   * 		position: Integer
   * 		code: String
   * }
   * require {
   * 		uri: String
   * 		data: TEXT
   * }
   * cache {
   * 		uri: String
   * 		data: Base64 encoded TEXT
   * }
   * values {
   * 		uri: String
   * 		values: TEXT
   * }
   */

  function dbError(reject) {
    return function (_t, e) {
      opera.postError('Database error: ' + e.message);
      if (e.code == 4) {
        tabsUtils.create('/notice_quota.html');
      }
      reject && reject();
    };
  }

  VMDB.prototype.openDB = function () {
    var _this = this;
    _this.db = openDatabase('Violentmonkey', '0.5', 'Violentmonkey data', 10 * 1024 * 1024);
    return _this.getTransaction(true).then(function (tx) {
      return [
        'CREATE TABLE IF NOT EXISTS scripts(id INTEGER PRIMARY KEY,uri VARCHAR,meta TEXT,custom TEXT,enabled INTEGER,"update" INTEGER,position INTEGER,code TEXT)',
        'CREATE TABLE IF NOT EXISTS cache(uri VARCHAR UNIQUE,data TEXT)',
        'CREATE TABLE IF NOT EXISTS require(uri VARCHAR UNIQUE,data TEXT)',
        'CREATE TABLE IF NOT EXISTS "values"(uri VARCHAR UNIQUE,data TEXT)',
      ].reduce(function (result, sql) {
        return result.then(function () {
          return new Promise(function (resolve, reject) {
            tx.executeSql(sql, [], function (_tx, res) {
              resolve(res);
            }, dbError(reject));
          });
        });
      }, Promise.resolve());
    });
  };

  VMDB.prototype.getTransaction = function (readwrite, tx) {
    var _this = this;
    return new Promise(function (resolve, reject) {
      if (tx) return resolve(tx);
      var method = readwrite ? 'transaction': 'readTransaction';
      _this.db[method].call(_this.db, function (tx) {
        resolve(tx);
      }, dbError(reject));
    });
  };

  VMDB.prototype.readSQLResult = function (res) {
    var data = [];
    var length = res.rows.length;
    for (var i = 0; i < length; i ++) {
      // item is read-only, so we clone it
      var item = res.rows.item(i);
      data.push(_.assign({}, item));
    }
    return data;
  };

  VMDB.prototype.initPosition = function () {
    var _this = this;
    _this.position = 0;
    return new Promise(function (resolve, reject) {
      _this.getTransaction().then(function (tx) {
        tx.executeSql('SELECT position FROM scripts ORDER BY position DESC LIMIT 1', [], function (_tx, res) {
          if (res.rows.length) _this.position = res.rows.item(0).position;
          resolve();
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.readScripts = function (res) {
    var scripts = this.readSQLResult(res);
    scripts.forEach(function (script) {
      if (typeof script.meta !== 'object') script.meta = JSON.parse(script.meta);
      if (typeof script.custom !== 'object') script.custom = JSON.parse(script.custom);
    });
    return scripts;
  };

  VMDB.prototype.getScript = function (id, tx) {
    var _this = this;
    return _this.getTransaction(false, tx).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('SELECT * FROM scripts WHERE id=? LIMIT 1', [id], function (_tx, res) {
          resolve(_this.readScripts(res)[0]);
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.queryScript = function (id, meta, tx) {
    var _this = this;
    return id
      ? _this.getScript(id, tx)
      : new Promise(function (resolve, reject) {
        var uri = scriptUtils.getNameURI({meta: meta});
        _this.getTransaction(false, tx).then(function (tx) {
          tx.executeSql('SELECT * FROM scripts WHERE uri=? LIMIT 1', [uri], function (_tx, res) {
            resolve(_this.readScripts(res)[0]);
          }, dbError(reject));
        });
      });
  };

  VMDB.prototype.getScriptData = function (id, tx) {
    return this.getScript(id, tx).then(function (script) {
      if (!script) return Promise.reject();
      var data = scriptUtils.getScriptInfo(script);
      data.code = script.code;
      return data;
    });
  };

  VMDB.prototype.getScriptInfos = function (ids) {
    var _this = this;
    return _this.getTransaction().then(function (tx) {
      return Promise.all(ids.map(function (id) {
        return _this.getScript(id, tx);
      }));
    }).then(function (scripts) {
      return scripts.filter(function (x) {return x;})
      .map(scriptUtils.getScriptInfo);
    });
  };

  VMDB.prototype.getValues = function (uris, tx) {
    var _this = this;
    return _this.getTransaction(false, tx).then(function (tx) {
      return uris.reduce(function (result, uri) {
        return result.then(function (data) {
          return new Promise(function (resolve, reject) {
            tx.executeSql('SELECT data FROM "values" WHERE uri=? LIMIT 1', [uri], function (_tx, res) {
              var item = _this.readSQLResult(res)[0];
              try {
                if (item) data[uri] = JSON.parse(item.data);
              } catch (e) {}
              resolve(data);
            }, dbError(reject));
          });
        });
      }, Promise.resolve({}));
    });
  };

  VMDB.prototype.getScriptsByURL = function (url) {
    function getScripts(tx) {
      return _this.getScriptsByIndex('position', null, tx).then(function (scripts) {
        var data = {
          uris: [],
        };
        var require = {};
        var cache = {};
        data.scripts = scripts.filter(function (script) {
          if (tester.testURL(url, script)) {
            data.uris.push(script.uri);
            script.meta.require.forEach(function (key) {
              require[key] = 1;
            });
            for (var k in script.meta.resources)
              cache[script.meta.resources[k]] = 1;
            return true;
          }
        });
        data.require = Object.keys(require);
        data.cache = Object.keys(cache);
        return data;
      });
    }
    function getRequire(uris, tx) {
      return uris.reduce(function (result, uri) {
        return result.then(function (data) {
          return new Promise(function (resolve, reject) {
            tx.executeSql('SELECT data FROM require WHERE uri=? LIMIT 1', [uri], function (_tx, res) {
              var item = _this.readSQLResult(res)[0];
              if (item) data[uri] = item.data;
              resolve(data);
            }, dbError(reject));
          });
        });
      }, Promise.resolve({}));
    }
    var _this = this;
    return _this.getTransaction().then(function (tx) {
      return getScripts(tx).then(function (data) {
        return Promise.all([
          getRequire(data.require, tx),
          _this.getValues(data.uris, tx),
          _this.getCacheB64(data.cache, tx),
        ]).then(function (res) {
          return {
            scripts: data.scripts,
            require: res[0],
            values: res[1],
            cache: res[2],
          };
        });
      });
    });
  };

  VMDB.prototype.getData = function () {
    function getScripts(tx) {
      return _this.getScriptsByIndex('position', null, tx).then(function (scripts) {
        var data = {};
        var cache = {};
        data.scripts = scripts.map(function (script) {
          var icon = script.meta.icon;
          if (scriptUtils.isRemote(icon)) cache[icon] = 1;
          return scriptUtils.getScriptInfo(script);
        });
        data.cache = Object.keys(cache);
        return data;
      });
    }
    function getCache(uris, tx) {
      return _this.getCacheB64(uris, tx).then(function (cache) {
        for (var k in cache)
          cache[k] = 'data:image/png;base64,' + cache[k];
        return cache;
      });
    }
    var _this = this;
    return _this.getTransaction().then(function (tx) {
      return getScripts(tx).then(function (data) {
        return getCache(data.cache, tx).then(function (cache) {
          return {
            scripts: data.scripts,
            cache: cache,
          };
        });
      });
    });
  };

  VMDB.prototype.removeScript = function (id) {
    return this.getTransaction(true).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('DELETE FROM scripts WHERE id=?', [id], function (_tx, _res) {
          resolve();
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.moveScript = function (id, offset) {
    var _this = this;
    return _this.getTransaction(true).then(function (tx) {
      return (new Promise(function (resolve, reject) {
        tx.executeSql('SELECT position FROM scripts WHERE id=?', [id], function (_tx, res) {
          resolve(_this.readSQLResult(res)[0].position);
        }, dbError(reject));
      })).then(function (pos) {
        return new Promise(function (resolve, reject) {
          var sql = 'SELECT id,position FROM scripts WHERE position' + (offset < 0 ? '<' : '>') + '? ORDER BY position ' + (offset < 0 ? 'DESC' : '') + ' LIMIT ?';
          if (offset < 0) offset = -offset;
          tx.executeSql(sql, [pos, offset], function (_tx, res) {
            var items = _this.readSQLResult(res);
            resolve([[items[items.length - 1].position, id]].concat(items.map(function (item, i) {
              return [i ? items[i - 1].position : pos, item.id];
            })));
          }, dbError(reject));
        });
      }).then(function (updates) {
        return updates.reduce(function (result, item) {
          return result.then(function () {
            return new Promise(function (resolve, reject) {
              tx.executeSql('UPDATE scripts SET position=? WHERE id=?', item, function (_tx, _res) {
                resolve();
              }, dbError(reject));
            });
          });
        }, Promise.resolve());
      });
    });
  };

  VMDB.prototype.getCacheB64 = function (uris, tx) {
    var _this = this;
    return _this.getTransaction(false, tx).then(function (tx) {
      return uris.reduce(function (result, uri) {
        return result.then(function (data) {
          return new Promise(function (resolve, reject) {
            tx.executeSql('SELECT data FROM cache WHERE uri=? LIMIT 1', [uri], function (_tx, res) {
              var item = _this.readSQLResult(res)[0];
              if (item) data[uri] = item.data;
              resolve(data);
            }, dbError(reject));
          });
        });
      }, Promise.resolve({}));
    });
  };

  VMDB.prototype.saveCache = function (url, data, tx) {
    return this.getTransaction(true, tx).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('REPLACE INTO cache(uri,data) VALUES(?,?)', [url, data], function (_tx, _res) {
          resolve();
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.saveRequire = function (url, data, tx) {
    return this.getTransaction(true, tx).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('REPLACE INTO require(uri,data) VALUES(?,?)', [url, data], function (_tx, _res) {
          resolve();
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.saveScript = function (script, tx) {
    script.enabled = script.enabled ? 1 : 0;
    script.update = script.update ? 1 : 0;
    if (!script.position) script.position = ++ this.position;
    return this.getTransaction(true, tx).then(function (tx) {
      return new Promise(function (resolve, reject) {
        var data = [
          +script.id || null,
          script.uri,
          JSON.stringify(script.meta),
          JSON.stringify(script.custom),
          script.enabled,
          script.update,
          script.position,
          script.code,
        ];
        tx.executeSql('REPLACE INTO scripts(id,uri,meta,custom,enabled,"update",position,code) VALUES(?,?,?,?,?,?,?,?)', data, function (_tx, res) {
          script.id = script.id || res.insertId;
          resolve(script);
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.fetchCache = function () {
    var requests = {};
    return function (url, check) {
      var _this = this;
      return requests[url]
      || (requests[url] = scriptUtils.fetch(url, 'blob').then(function (res) {
        return (check ? check(res.response) : Promise.resolve()).then(function () {
          return res.response;
        });
      }).then(function (data) {
        return new Promise(function (resolve, reject) {
          var reader = new FileReader;
          reader.onload = function (_e) {
            _this.saveCache(url, window.btoa(this.result)).then(function () {
              delete requests[url];
              resolve();
            });
          };
          reader.onerror = function (e) {
            reject(e);
          };
          reader.readAsBinaryString(data);
        });
      }));
    };
  }();

  VMDB.prototype.fetchRequire = function () {
    var requests = {};
    return function (url) {
      var _this = this;
      var promise = requests[url];
      if (!promise) {
        promise = requests[url] = scriptUtils.fetch(url)
        .then(function (res) {
          return _this.saveRequire(url, res.responseText);
        })
        .catch(function () {
          opera.postError('Error fetching required script: ' + url);
        })
        .then(function () {
          delete requests[url];
        });
      }
      return promise;
    };
  }();

  VMDB.prototype.setValue = function (uri, values) {
    return this.getTransaction(true).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('REPLACE INTO "values"(uri,data) VALUES(?,?)', [uri, JSON.stringify(values)], function (_tx, _res) {
          resolve();
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.updateScriptInfo = function (id, data) {
    var _this = this;
    return _this.getTransaction(true).then(function (tx) {
      return _this.getScript(id, tx).then(function (script) {
        var updates = [];
        var args = [];
        for (var k in data) {
          var v = data[k];
          if (~['enabled', 'update'].indexOf(k)) {
            v = v ? 1 : 0;
          } else continue;
          updates.push('"' + k + '"=?');
          args.push(v);
          script[k] = v;
        }
        args.push(id);
        return new Promise(function (resolve, reject) {
          tx.executeSql('UPDATE scripts SET ' + updates.join(',') + ' WHERE id=?', args, function (_tx, _res) {
            resolve(scriptUtils.getScriptInfo(script));
          }, dbError(reject));
        });
      });
    });
  };

  VMDB.prototype.getExportData = function (ids, withValues) {
    function getScripts(ids, tx) {
      return Promise.all(ids.map(function (id) {
        return _this.getScriptData(id, tx);
      })).then(function (data) {
        return data.filter(function (x) {return x;});
      });
    }
    var _this = this;
    return _this.getTransaction().then(function (tx) {
      return getScripts(ids, tx).then(function (scripts) {
        var res = {
          scripts: scripts,
        };
        return withValues
        ? _this.getValues(scripts.map(function (script) {
          return script.uri;
        }), tx).then(function (values) {
          res.values = values;
          return res;
        }) : res;
      });
    });
  };

  VMDB.prototype.vacuum = function () {
    function getScripts(tx) {
      return _this.getScriptsByIndex('position', null, tx).then(function (scripts) {
        var data = {
          require: {},
          cache: {},
          values: {},
        };
        data.ids = scripts.map(function (script) {
          script.meta.require.forEach(function (uri) {data.require[uri] = 1;});
          for (var k in script.meta.resources)
            data.cache[script.meta.resources[k]] = 1;
          if (scriptUtils.isRemote(script.meta.icon))
            data.cache[script.meta.icon] = 1;
          data.values[script.uri] = 1;
          return script.id;
        });
        return data;
      });
    }
    function vacuumPosition(ids, tx) {
      return ids.reduce(function (result, id, i) {
        return result.then(function () {
          return new Promise(function (resolve, reject) {
            tx.executeSql('UPDATE scripts SET position=? WHERE id=?', [i + 1, id], function (_tx, _res) {
              resolve();
            }, dbError(reject));
          });
        });
      }, Promise.resolve());
    }
    function vacuumCache(dbName, dict, tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('SELECT uri FROM "' + dbName + '"', [], function (tx, res) {
          resolve(_this.readSQLResult(res).reduce(function (result, item) {
            return result.then(function () {
              return new Promise(function (resolve, reject) {
                if (dict[item.uri]) resolve(dict[item.uri] ++);
                else tx.executeSql('DELETE FROM "' + dbName + '" WHERE uri=?', [item.uri], function (_tx, _res) {
                  resolve();
                }, dbError(reject));
              });
            });
          }, Promise.resolve()));
        }, dbError(reject));
      });
    }
    var _this = this;
    return _this.getTransaction(true).then(function (tx) {
      return getScripts(tx).then(function (data) {
        return Promise.all([
          vacuumPosition(data.ids, tx),
          vacuumCache('require', data.require, tx),
          vacuumCache('cache', data.cache, tx),
          vacuumCache('values', data.values, tx),
        ]).then(function () {
          return {
            require: data.require,
            cache: data.cache,
          };
        });
      }).then(function (data) {
        return Promise.all([
          Object.keys(data.require).map(function (k) {
            return data.require[k] === 1 && _this.fetchRequire(k);
          }),
          Object.keys(data.cache).map(function (k) {
            return data.cache[k] === 1 && _this.fetchCache(k);
          }),
        ]);
      });
    });
  };

  VMDB.prototype.getScriptsByIndex = function (index, cond, tx) {
    var _this = this;
    return _this.getTransaction(false, tx).then(function (tx) {
      return new Promise(function (resolve, reject) {
        tx.executeSql('SELECT * FROM scripts' + (cond ? ' WHERE ' + cond : '') + ' ORDER BY "' + index + '"', [], function (_tx, res) {
          resolve(_this.readScripts(res));
        }, dbError(reject));
      });
    });
  };

  VMDB.prototype.parseScript = function (data) {
    var res = {
      cmd: 'update',
      data: {
        message: data.message == null ? _.i18n('msgUpdated') : data.message || '',
      },
    };
    var meta = scriptUtils.parseMeta(data.code);
    var _this = this;
    return _this.getTransaction(true).then(function (tx) {
      // @require
      meta.require.forEach(function (url) {
        var cache = data.require && data.require[url];
        cache ? _this.saveRequire(url, cache, tx) : _this.fetchRequire(url);
      });
      // @resource
      Object.keys(meta.resources).forEach(function (k) {
        var url = meta.resources[k];
        var cache = data.resources && data.resources[url];
        cache ? _this.saveCache(url, cache, tx) : _this.fetchCache(url);
      });
      // @icon
      if (scriptUtils.isRemote(meta.icon))
        _this.fetchCache(meta.icon, function (blob) {
          return new Promise(function (resolve, reject) {
            var reader = new FileReader;
            reader.onload = function (e) {
              resolve(e.target.result);
            };
            reader.onerror = function () {
              reject();
            };
            reader.readAsDataURL(blob);
          }).then(function (dataURL) {
            return new Promise(function (resolve, reject) {
              var image = new Image;
              image.onload = function () {
                resolve(blob);
              };
              image.onerror = function () {
                reject();
              };
              image.src = dataURL;
            });
          });
        });
      return _this.queryScript(data.id, meta, tx).then(function (script) {
        if (script) {
          if (data.isNew) throw _.i18n('msgNamespaceConflict');
        } else {
          script = scriptUtils.newScript();
          res.cmd = 'add';
          res.data.message = _.i18n('msgInstalled');
        }
        if (data.more) for (var k in data.more)
          if (k in script) script[k] = data.more[k];
        script.meta = meta;
        script.code = data.code;
        script.uri = scriptUtils.getNameURI(script);
        // use referer page as default homepage
        if (!meta.homepageURL && !script.custom.homepageURL && scriptUtils.isRemote(data.from))
          script.custom.homepageURL = data.from;
        if (scriptUtils.isRemote(data.url))
          script.custom.lastInstallURL = data.url;
        script.custom.modified = data.modified || Date.now();
        return _this.saveScript(script, tx);
      });
    }).then(function (script) {
      _.assign(res.data, scriptUtils.getScriptInfo(script));
      return res;
    });
  };

  VMDB.prototype.checkUpdate = function () {
    function check(script) {
      var res = {
        cmd: 'update',
        data: {
          id: script.id,
          checking: true,
        },
      };
      var downloadURL = script.custom.downloadURL || script.meta.downloadURL || script.custom.lastInstallURL;
      var updateURL = script.custom.updateURL || script.meta.updateURL || downloadURL;
      var okHandler = function (xhr) {
        var meta = scriptUtils.parseMeta(xhr.responseText);
        if (scriptUtils.compareVersion(script.meta.version, meta.version) < 0)
          return Promise.resolve();
        res.data.checking = false;
        res.data.message = _.i18n('msgNoUpdate');
        _.messenger.post(res);
        return Promise.reject();
      };
      var errHandler = function (_xhr) {
        res.data.checking = false;
        res.data.message = _.i18n('msgErrorFetchingUpdateInfo');
        _.messenger.post(res);
        return Promise.reject();
      };
      var update = function () {
        if (!downloadURL) {
          res.data.message = '<span class="new">' + _.i18n('msgNewVersion') + '</span>';
          _.messenger.post(res);
          return Promise.reject();
        }
        res.data.message = _.i18n('msgUpdating');
        _.messenger.post(res);
        return scriptUtils.fetch(downloadURL).then(function (xhr) {
          return xhr.responseText;
        }, function (_xhr) {
          res.data.checking = false;
          res.data.message = _.i18n('msgErrorFetchingScript');
          _.messenger.post(res);
          return Promise.reject();
        });
      };
      if (!updateURL) return Promise.reject();
      res.data.message = _.i18n('msgCheckingForUpdate');
      _.messenger.post(res);
      return scriptUtils.fetch(updateURL, null, {
        Accept: 'text/x-userscript-meta',
      }).then(okHandler, errHandler).then(update);
    }

    var processes = {};
    return function (script) {
      var _this = this;
      var promise = processes[script.id];
      if (!promise)
        promise = processes[script.id] = check(script).then(function (code) {
          delete processes[script.id];
          return _this.parseScript({
            id: script.id,
            code: code,
          }).then(function (res) {
            res.data.checking = false;
            _.messenger.post(res);
          });
        }, function () {
          delete processes[script.id];
          //return Promise.reject();
        });
      return promise;
    };
  }();

  module.exports = VMDB;
});

define('requests', function (_require, _exports, module) {
  function getRequestId() {
    var id = _.getUniqId();
    requests[id] = {
      id: id,
      xhr: new XMLHttpRequest,
    };
    return id;
  }
  function xhrCallbackWrapper(req) {
    var lastPromise = Promise.resolve();
    var xhr = req.xhr;
    return function (evt) {
      var res = {
        id: req.id,
        type: evt.type,
        resType: xhr.responseType,
      };
      var data = res.data = {
        readyState: xhr.readyState,
        responseHeaders: xhr.getAllResponseHeaders(),
        status: xhr.status,
        statusText: xhr.statusText,
      };
      try {
        data.responseText = xhr.responseText;
      } catch (e) {}
      if (evt.type === 'loadend') clearRequest(req);
      return lastPromise = lastPromise.then(function () {
        return new Promise(function (resolve, _reject) {
          if (xhr.response && xhr.responseType === 'blob') {
            var reader = new FileReader;
            reader.onload = function (_e) {
              data.response = this.result;
              resolve();
            };
            reader.readAsDataURL(xhr.response);
          } else {
            // default `null` for blob and '' for text
            data.response = xhr.response;
            resolve();
          }
        });
      }).then(function () {
        req.cb && req.cb(res);
      });
    };
  }
  function httpRequest(details, cb) {
    var req = requests[details.id];
    if (!req || req.cb) return;
    req.cb = cb;
    var xhr = req.xhr;
    try {
      xhr.open(details.method, details.url, true, details.user, details.password);
      if (details.headers) {
        for (var k in details.headers) {
          xhr.setRequestHeader(k, details.headers[k]);
        }
      }
      if (details.responseType)
        xhr.responseType = 'blob';
      if (details.overrideMimeType)
        xhr.overrideMimeType(details.overrideMimeType);
      var callback = xhrCallbackWrapper(req);
      [
        'abort',
        'error',
        'load',
        'loadend',
        'progress',
        'readystatechange',
        'timeout',
      ].forEach(function (evt) {
        xhr['on' + evt] = callback;
      });
      xhr.send(details.data);
    } catch (e) {
      opera.postError(e.toString());
    }
  }
  function abortRequest(id) {
    var req = requests[id];
    if (req) {
      req.xhr.abort();
      clearRequest(req);
    }
  }
  function clearRequest(req) {
    delete requests[req.id];
  }

  var requests = {};
  module.exports = {
    getRequestId: getRequestId,
    abortRequest: abortRequest,
    httpRequest: httpRequest,
  };
});

define('utils/cache', function (_require, _exports, module) {
  function get(key) {
    var obj = cache[key];
    return obj && obj.value;
  }
  function set(key, value) {
    if (value) {
      var obj = cache[key];
      if (!obj) obj = cache[key] = {
        key: key,
      };
      obj.value = value;
      if (obj.timer) clearTimeout(obj.timer);
      obj.timer = setTimeout(function () {
        set(key);
      }, 3000);
    } else {
      delete cache[key];
    }
  }
  var cache = {};
  module.exports = {
    get: get,
    set: set,
  };
});

define('utils/i18n', function (_require, _exports, module) {
	function getData() {
		var xhr = new XMLHttpRequest;
		xhr.open('GET', '/messages.json', true);
		xhr.responseType = 'json';
		xhr.onload = function () {
			messages = this.response;
		};
		xhr.onerror = function () {
			// Though this should not happen, it did happened!
			setTimeout(getData, 500);
		};
		xhr.send();
	}
	var messages = {};
	getData();
	module.exports = function (key, args) {
		if (!key) return '';
		var value = messages[key];
		var data = '';
		if (value) {
			args = args || [];
			args.unshift(key);
			data = value.message.replace(/\$(?:\{(\d+)\}|(\d+))/g, function(match, group1, group2) {
				var arg = args[group1 || group2];
				return arg == null ? match : arg;
			});
		}
		return data;
	};
});

define('utils/script', function (_require, _exports, module) {
  var scriptUtils = module.exports = {
    isRemote: function (url) {
      return url && !(/^(file|data):/.test(url));
    },
    fetch: function (url, type, headers) {
      var xhr = new XMLHttpRequest;
      xhr.open('GET', url, true);
      if (type) xhr.responseType = type;
      if (headers) for (var k in headers) {
        xhr.setRequestHeader(k, headers[k]);
      }
      return new Promise(function (resolve, reject) {
        xhr.onloadend = function () {
          (xhr.status > 300 ? reject : resolve)(xhr);
        };
        xhr.send();
      });
    },
    parseMeta: function (code) {
      // initialize meta, specify those with multiple values allowed
      var meta = {
        include: [],
        exclude: [],
        match: [],
        require: [],
        resource: [],
        grant: [],
      };
      var flag = -1;
      code.replace(/(?:^|\n)\/\/\s*([@=]\S+)(.*)/g, function (value, group1, group2) {
        if (flag < 0 && group1 == '==UserScript==') {
          // start meta
          flag = 1;
        } else if (flag > 0 && group1 == '==/UserScript==') {
          // end meta
          flag = 0;
        }
        if (flag == 1 && group1[0] == '@') {
          var key = group1.slice(1);
          var val = group2.replace(/^\s+|\s+$/g, '');
          value = meta[key];
          // multiple values allowed
          if (value && value.push) value.push(val);
          // only first value will be stored
          else if (!(key in meta)) meta[key] = val;
        }
      });
      meta.resources = {};
      meta.resource.forEach(function (line) {
        var pair = line.match(/^(\w\S*)\s+(.*)/);
        if (pair) meta.resources[pair[1]] = pair[2];
      });
      delete meta.resource;
      // @homepageURL: compatible with @homepage
      if (!meta.homepageURL && meta.homepage) meta.homepageURL = meta.homepage;
      return meta;
    },
    newScript: function () {
      var script = {
        custom: {},
        enabled: 1,
        update: 1,
        code: '// ==UserScript==\n// @name New Script\n// @namespace Violentmonkey Scripts\n// @grant none\n// ==/UserScript==\n',
      };
      script.meta = scriptUtils.parseMeta(script.code);
      return script;
    },
    getScriptInfo: function (script) {
      return {
        id: script.id,
        custom: script.custom,
        meta: script.meta,
        enabled: script.enabled,
        update: script.update,
      };
    },
    getNameURI: function (script) {
      var ns = script.meta.namespace || '';
      var name = script.meta.name || '';
      var nameURI = escape(ns) + ':' + escape(name) + ':';
      if (!ns && !name) nameURI += script.id || '';
      return nameURI;
    },
    compareVersion: function (ver1, ver2) {
      ver1 = (ver1 || '').split('.');
      ver2 = (ver2 || '').split('.');
      var len1 = ver1.length, len2 = ver2.length;
      for (var i = 0; i < len1 || i < len2; i ++) {
        var delta = (parseInt(ver1[i], 10) || 0) - (parseInt(ver2[i], 10) || 0);
        if (delta) return delta < 0 ? -1 : 1;
      }
      return 0;
    },
  };
});

define('utils/tabs', function (_require, _exports, module) {
  module.exports = {
    create: function (url) {
      opera.extension.tabs.create({url: url}).focus();
    },
    get: function () {
      return opera.extension.tabs.getFocused();
    },
  };
});

define('utils/tester', function (_require, exports, _module) {
  function testURL(url, script) {
    var custom = script.custom;
    var meta = script.meta;
    var inc = [], exc = [], mat = [];
    if (custom._match !== false && meta.match) mat = mat.concat(meta.match);
    if (custom.match) mat = mat.concat(custom.match);
    if (custom._include !== false && meta.include) inc = inc.concat(meta.include);
    if (custom.include) inc = inc.concat(custom.include);
    if (custom._exclude !== false && meta.exclude) exc = exc.concat(meta.exclude);
    if (custom.exclude) exc = exc.concat(custom.exclude);
    var ok = !mat.length && !inc.length;
    // @match
    ok = ok || mat.length && function (test) {
      return mat.some(test);
    }(matchTester(url));
    // @include
    ok = ok || inc.some(function (str) {
      return autoReg(str).test(url);
    });
    // exclude
    ok = ok && !exc.some(function (str) {
      return autoReg(str).test(url);
    });
    return ok;
  }

  function str2RE(str) {
    return RegExp('^' + str.replace(/([.?\/])/g, '\\$1').replace(/\*/g, '.*?') + '$');
  }

  function autoReg(str) {
    if (/^\/.*\/$/.test(str))
      return RegExp(str.slice(1, -1));  // Regular-expression
    else
      return str2RE(str); // String with wildcards
  }

  function matchTester(url) {
    function matchScheme(rule, data) {
      // exact match
      if (rule == data) return 1;
      // * = http | https
      if (rule == '*' && /^https?$/i.test(data)) return 1;
      return 0;
    }
    function matchHost(rule, data) {
      // * matches all
      if (rule == '*') return 1;
      // exact match
      if (rule == data) return 1;
      // *.example.com
      if (/^\*\.[^*]*$/.test(rule)) {
        // matches the specified domain
        if (rule.slice(2) == data) return 1;
        // matches subdomains
        if (str2RE(rule).test(data)) return 1;
      }
      return 0;
    }
    function matchPath(rule, data) {
      return str2RE(rule).test(data);
    }
    var RE = /(.*?):\/\/([^\/]*)\/(.*)/;
    var urlParts = url.match(RE);
    return function (str) {
      if (str == '<all_urls>') return true;
      var parts = str.match(RE);
      return !!parts
        && matchScheme(parts[1], urlParts[1])
        && matchHost(parts[2], urlParts[2])
        && matchPath(parts[3], urlParts[3]);
    };
  }

  exports.testURL = testURL;
});
;define.use("app");