define('app', function (require, exports, _module) {
  var MainView = require('views/Main');
  var ConfirmView = require('views/Confirm');
  var models = require('models');
  var cache = require('cache');

  var App = cache.BaseRouter.extend({
    routes: {
      '': 'renderMain',
      'main/:tab': 'renderMain',
      'confirm/:url': 'renderConfirm',
      'confirm/:url/:from': 'renderConfirm',
    },
    renderMain: function (tab) {
      this.loadView('main', function () {
        initMain();
        return new MainView;
      }).loadTab(tab);
    },
    renderConfirm: function (url, referer) {
      this.loadView('confirm', function () {
        return new ConfirmView;
      }).initData(url, referer);
    },
  });
  var app = new App('#app');
  Backbone.history.start() || app.navigate('', {trigger: true, replace: true});

  function initMain() {
    var scriptList = exports.scriptList = new models.ScriptList;
    _.bg._.messenger.connect(window, function (res) {
      if (res.cmd === 'add') {
        res.data.message = '';
        scriptList.push(res.data);
      } else if (res.data) {
        var model = scriptList.get(res.data.id);
        if (model) model.set(res.data);
      }
    });
  }
});

define('editor', function (_require, exports, _module) {
  function addScripts(data) {
    function add(data) {
      var s = document.createElement('script');
      if (data.innerHTML) s.innerHTML = data.innerHTML;
      else if (data.src) s.src = data.src;
      return new Promise(function (resolve, reject) {
        s.onload = function () {
          resolve();
        };
        s.onerror = function () {
          reject();
        };
        document.body.appendChild(s);
      });
    }
    if (!data.map) data = [data];
    return Promise.all(data.map(add));
  }

  function addCSS(data) {
    function add(data) {
      var s;
      if (data.html) {
        s = document.createElement('style');
        s.innerHTML = data.html;
      } else if (data.href) {
        s = document.createElement('link');
        s.rel = 'stylesheet';
        s.href = data.href;
      }
      if (s) document.body.appendChild(s);
    }
    if (!data.forEach) data = [data];
    data.forEach(add);
  }

  function initCodeMirror() {
    addCSS([
      {href: '/lib/CodeMirror/lib/codemirror.css'},
      {href: '/mylib/CodeMirror/fold.css'},
      {href: '/mylib/CodeMirror/search.css'},
    ]);
    return addScripts(
      {src: '/lib/CodeMirror/lib/codemirror.js'}
    ).then(function () {
      return addScripts([
        {src: '/lib/CodeMirror/mode/javascript/javascript.js'},
        {src: '/lib/CodeMirror/addon/comment/continuecomment.js'},
        {src: '/lib/CodeMirror/addon/edit/matchbrackets.js'},
        {src: '/lib/CodeMirror/addon/edit/closebrackets.js'},
        {src: '/lib/CodeMirror/addon/fold/foldcode.js'},
        {src: '/lib/CodeMirror/addon/fold/foldgutter.js'},
        {src: '/lib/CodeMirror/addon/fold/brace-fold.js'},
        {src: '/lib/CodeMirror/addon/fold/comment-fold.js'},
        {src: '/lib/CodeMirror/addon/search/match-highlighter.js'},
        {src: '/lib/CodeMirror/addon/search/searchcursor.js'},
        {src: '/lib/CodeMirror/addon/selection/active-line.js'},
        {src: '/mylib/CodeMirror/search.js'},
      ]);
    });
  }

  var readyCodeMirror;

  exports.init = function (options) {
    options = options || {};
    readyCodeMirror = readyCodeMirror || initCodeMirror();
    return readyCodeMirror.then(function(){
      var editor = CodeMirror(options.container, {
        continueComments: true,
        matchBrackets: true,
        autoCloseBrackets: true,
        highlightSelectionMatches: true,
        lineNumbers: true,
        mode: 'javascript',
        lineWrapping: true,
        styleActiveLine: true,
        foldGutter: true,
        gutters: ['CodeMirror-linenumbers', 'CodeMirror-foldgutter'],
      });
      editor.clearHistory = function() {
        this.getDoc().clearHistory();
      };
      editor.setValueAndFocus = function(value) {
        this.setValue(value);
        this.focus();
      };
      options.readonly && editor.setOption('readOnly', options.readonly);
      options.onchange && editor.on('change', options.onchange);
      var extraKeys = {};
      options.onexit && (extraKeys.Esc = options.onexit);
      options.onsave && (extraKeys['Ctrl-S'] = extraKeys['Cmd-S'] = options.onsave);
      editor.setOption('extraKeys', extraKeys);
      return editor;
    });
  };
});

define('models', function (_require, exports, _module) {
  exports.Meta = Backbone.Model.extend({
    parse: function (script) {
      this.meta = script.meta;
      return script.custom;
    },
  });

  exports.Script = Backbone.Model.extend({
    getLocaleString: function (key) {
      var _this = this;
      var meta = _this.get('meta') || {};
      return _.getLocaleString(meta, key);
    },
    canUpdate: function () {
      var script = this.toJSON();
      return script.update && (
        script.custom.updateURL ||
          script.meta.updateURL ||
          script.custom.downloadURL ||
          script.meta.downloadURL ||
          script.custom.lastInstallURL
      );
    },
  });

  exports.ScriptList = Backbone.Collection.extend({
    model: exports.Script,
    // comparator: 'position',
    initialize: function () {
      this.cache = {};
      this.reload();
    },
    reload: function () {
      var _this = this;
      _this.loading = true;
      _.sendMessage({cmd: 'GetData'}).then(function (data) {
        _this.loading = false;
        _.assign(_this.cache, data.cache);
        _this.reset(data.scripts);
      });
    },
  });
});

define('views/ConfirmOption', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;

  module.exports = BaseView.extend({
    className: 'button-panel options-panel',
    events: {
      'mousedown': 'stopPropagation',
      'change [data-check]': 'updateCheckbox',
      'change #cbClose': 'render',
    },
    templateUrl: '/options/templates/confirm-options.html',
    _render: function () {
      var options = _.options.getAll();
      this.$el.html(this.templateFn(options));
    },
    stopPropagation: function (e) {
      e.stopPropagation();
    },
    updateCheckbox: _.updateCheckbox,
  });
});

define('views/Confirm', function (require, _exports, module) {
  var cache = _.require('utils/cache');
  var tabs = _.require('utils/tabs');
  var BaseView = require('cache').BaseView;
  var ConfirmOptionsView = require('views/ConfirmOption');
  var editor = require('editor');

  module.exports = BaseView.extend({
    events: {
      'click .button-toggle': 'toggleOptions',
      'click #btnInstall': 'installScript',
      'click #btnClose': 'close',
    },
    templateUrl: '/options/templates/confirm.html',
    initialize: function () {
      var _this = this;
      _.bindAll(_this, 'hideOptions', 'trackLocalFile');
      BaseView.prototype.initialize.call(_this);
    },
    _render: function () {
      var _this = this;
      _this.$el.html(_this.templateFn());
      _this.loadedEditor = editor.init({
        container: _this.$('.editor-code')[0],
        readonly: true,
        onexit: _this.close,
      }).then(function (editor) {
        _this.editor = editor;
      });
      _this.$toggler = _this.$('.button-toggle');
      _this.$('#url').attr('title', _this.url).text(_this.url);
      _this.showMessage(_.i18n('msgLoadingData'));
    },
    initData: function (url, referer) {
      var _this = this;
      _this.url = url;
      _this.from = referer;
      _this.render();
      _this.loadData().then(function () {
        _this.parseMeta();
      });
    },
    loadData: function (changedOnly) {
      var _this = this;
      var _text = _this.data && _this.data.code;
      _this.$('#btnInstall').prop('disabled', true);
      _this.data = {
        code: _text,
        require: {},
        resources: {},
        dependencyOK: false,
        isLocal: /^file:\/\/\//.test(_this.url),
      };
      return _this.getScript(_this.url).then(function (text) {
        if (changedOnly && _text == text) return Promise.reject();
        _this.data.code = text;
        _this.loadedEditor.then(function () {
          _this.editor.setValueAndFocus(_this.data.code);
        });
      });
    },
    parseMeta: function () {
      var _this = this;
      return _.sendMessage({
        cmd: 'ParseMeta',
        data: _this.data.code,
      }).then(function (script) {
        var urls = _.values(script.resources);
        var length = script.require.length + urls.length;
        var finished = 0;
        if (!length) return;
        var error = [];
        var updateStatus = function () {
          _this.showMessage(_.i18n('msgLoadingDependency', [finished, length]));
        };
        updateStatus();
        var promises = script.require.map(function (url) {
          return _this.getFile(url).then(function (res) {
            _this.data.require[url] = res;
          });
        });
        promises = promises.concat(urls.map(function (url) {
          return _this.getFile(url, true).then(function (res) {
            _this.data.resources[url] = res;
          });
        }));
        promises = promises.map(function (promise) {
          return promise.then(function () {
            finished += 1;
            updateStatus();
          }, function (url) {
            error.push(url);
          });
        });
        return Promise.all(promises).then(function () {
          if (error.length) return Promise.reject(error.join('\n'));
          _this.data.dependencyOK = true;
        });
      }).then(function () {
        _this.showMessage(_.i18n('msgLoadedData'));
        _this.$('#btnInstall').prop('disabled', false);
      }, function (error) {
        _this.showMessage(_.i18n('msgErrorLoadingDependency'), error);
        return Promise.reject();
      });
    },
    hideOptions: function () {
      var _this = this;
      if (!_this.optionsView) return;
      _this.$toggler.removeClass('active');
      _this.optionsView.remove();
      _this.optionsView = null;
    },
    toggleOptions: function (_e) {
      var _this = this;
      if (_this.optionsView) {
        _this.hideOptions();
      } else {
        _this.$toggler.addClass('active');
        _this.optionsView = new ConfirmOptionsView;
        _this.optionsView.$el.insertAfter(_this.$toggler);
        $(document).one('mousedown', _this.hideOptions);
      }
    },
    close: function () {
      tabs.get().close();
    },
    showMessage: function (msg) {
      this.$('#msg').html(msg);
    },
    getFile: function (url, isBlob) {
      return new Promise(function (resolve, reject) {
        var xhr = new _.bg.XMLHttpRequest;
        xhr.open('GET', url, true);
        if (isBlob) xhr.responseType = 'blob';
        xhr.onloadend = function () {
          if (xhr.status > 300) return reject(url);
          if (isBlob) {
            var reader = new FileReader;
            reader.onload = function (_e) {
              resolve(window.btoa(this.result));
            };
            reader.readAsBinaryString(xhr.response);
          } else {
            resolve(xhr.responseText);
          }
        };
        xhr.send();
      });
    },
    getScript: function (url) {
      var _this = this;
      return new Promise(function (resolve, reject) {
        var text = cache.get(url);
        return text ? resolve(text) : reject();
      }).catch(function () {
        return _this.getFile(url)
        .catch(function (url) {
          _this.showMessage(_.i18n('msgErrorLoadingData'));
          throw url;
        });
      });
    },
    getTimeString: function () {
      var now = new Date();
      return _.zfill(now.getHours(), 2) + ':' +
        _.zfill(now.getMinutes(), 2) + ':' +
        _.zfill(now.getSeconds(), 2);
    },
    installScript: function () {
      var _this = this;
      _this.$('#btnInstall').prop('disabled', true);
      _.sendMessage({
        cmd:'ParseScript',
        data:{
          url: _this.url,
          from: _this.from,
          code: _this.data.code,
          require: _this.data.require,
          resources: _this.data.resources,
        },
      }).then(function (res) {
        _this.showMessage(res.message + '[' + _this.getTimeString() + ']');
        if (res.code < 0) return;
        if (_.options.get('closeAfterInstall')) _this.close();
        else if (_this.data.isLocal && _.options.get('trackLocalFile')) _this.trackLocalFile();
      });
    },
    trackLocalFile: function () {
      var _this = this;
      setTimeout(function () {
        _this.loadData(true).then(function () {
          var track = _.options.get('trackLocalFile');
          if (!track) return;
          _this.parseMeta().then(function () {
            track && _this.installScript();
          });
        }, _this.trackLocalFile);
      }, 2000);
    },
  });
});

define('views/Meta', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;

  module.exports = BaseView.extend({
    className: 'button-panel',
    templateUrl: '/options/templates/edit-meta.html',
    events: {
      'change [data-id]': 'onChange',
      'mousedown': 'onMousedown',
    },
    _render: function () {
      var model = this.model;
      var it = model.toJSON();
      it.__name = model.meta.name;
      it.__homepageURL = model.meta.homepageURL;
      it.__updateURL = model.meta.updateURL || _.i18n('hintUseDownloadURL');
      it.__downloadURL = model.meta.downloadURL || it.lastInstallURL;
      this.$el.html(this.templateFn(it));
    },
    onChange: function (e) {
      e.stopPropagation();
      var res = this.getValue(e.target);
      this.model.set(res.key, res.value);
    },
    onMousedown: function (e) {
      e.stopPropagation();
    },
  });
});

define('views/Edit', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var Message = require('views/Message');
  var MetaView = require('views/Meta');
  var Meta = require('models').Meta;
  var app = require('app');
  var editor = require('editor');

  module.exports = BaseView.extend({
    className: 'frame edit',
    templateUrl: '/options/templates/edit.html',
    events: {
      'click .button-toggle': 'toggleMeta',
      'change [data-id]': 'updateCheckbox',
      'click #editorSave': 'save',
      'click #editorClose': 'close',
      'click #editorSaveClose': 'saveClose',
    },
    initialize: function () {
      var _this = this;
      BaseView.prototype.initialize.call(_this);
      _this.metaModel = new Meta(_this.model.toJSON(), {parse: true});
      _this.listenTo(_this.metaModel, 'change', function (model) {
        _this.model.set('custom', model.toJSON());
      });
      _this.listenTo(_this.model, 'change', function (_model) {
        _this.updateStatus(true);
      });
      _.bindAll(_this, 'save', 'close', 'hideMeta');
    },
    _render: function () {
      var _this = this;
      var it = _this.model.toJSON();
      _this.$el.html(_this.templateFn(it));
      _this.$toggler = _this.$('.button-toggle');
      var gotScript = it.id ? _.sendMessage({
        cmd: 'GetScript',
        data: it.id,
      }) : Promise.resolve(it);
      _this.loadedEditor = new Promise(function (resolve, _reject) {
        // Fix Opera: delayed so that the DOM is ready for building editor
        setTimeout(resolve);
      }).then(function () {
        return editor.init({
          container: _this.$('.editor-code')[0],
          onsave: _this.save,
          onexit: _this.close,
          onchange: function (_e) {
            _this.model.set('code', _this.editor.getValue());
          },
        });
      });
      Promise.all([
        gotScript,
        _this.loadedEditor,
      ]).then(function (res) {
        var script = res[0];
        var editor = _this.editor = res[1];
        editor.setValueAndFocus(script.code);
        editor.clearHistory();
        _this.updateStatus(false);
      });
    },
    updateStatus: function (changed) {
      this.changed = changed;
      this.$('#editorSave').prop('disabled', !changed);
      this.$('#editorSaveClose').prop('disabled', !changed);
    },
    save: function () {
      var _this = this;
      var data = _this.model.toJSON();
      return _.sendMessage({
        cmd: 'ParseScript',
        data: {
          id: data.id,
          code: data.code,
          isNew: !data.id,
          message: '',
          more: {
            custom: data.custom,
            update: data.update,
          }
        }
      }).then(function (script) {
        _this.model.set('id', script.id);
        _this.updateStatus(false);
      }, function (err) {
        new Message({
          data: err,
        });
      });
    },
    close: function () {
      if (!this.changed || confirm(_.i18n('confirmNotSaved')))
        app.scriptList.trigger('edit:close');
    },
    saveClose: function () {
      this.save().then(this.close.bind(this));
    },
    hideMeta: function () {
      if (!this.metaView) return;
      this.$toggler.removeClass('active');
      this.metaView.remove();
      this.metaView = null;
    },
    toggleMeta: function (e) {
      if (this.metaView) {
        this.hideMeta();
      } else {
        this.$toggler.addClass('active');
        this.metaView = new MetaView({model: this.metaModel});
        this.metaView.$el.insertAfter(e.target);
        $(document).one('mousedown', this.hideMeta);
      }
    },
    updateCheckbox: function (e) {
      var res = this.getValue(e.target);
      this.model.set(res.key, res.value);
    },
  });
});

define('views/Main', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var MainTab = require('views/TabMain');
  var SettingsTab = require('views/TabSettings');
  var AboutTab = require('views/TabAbout');
  module.exports = BaseView.extend({
    className: 'main',
    templateUrl: '/options/templates/main.html',
    tabs: {
      main: MainTab,
      settings: SettingsTab,
      about: AboutTab,
    },
    initialize: function () {
      var _this = this;
      _this.model = new Backbone.Model({
        tab: null,
      });
      _this.listenTo(_this.model, 'change', _this.render);
      BaseView.prototype.initialize.call(_this);
    },
    _render: function () {
      var _this = this;
      var Tab = _this.model.get('tab');
      /* Empty check needed for sync-promise */
      if (!Tab) return;
      var name = Tab.prototype.name;
      _this.$el.html(_this.templateFn({tab: name}));
      _this.loadSubview(name, function () {
        return new Tab;
      }, '#tab');
    },
    loadTab: function (name) {
      var _this = this;
      _this.model.set('tab', _this.tabs[name] || _this.tabs['main']);
    },
  });
});

define('views/Message', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  module.exports = BaseView.extend({
    className: 'message',
    templateUrl: '/options/templates/message.html',
    transitionTime: 500,
    initialize: function (options) {
      var _this = this;
      _this.options = options;
      _.bindAll(_this, 'toggle', 'delay', 'remove');
      BaseView.prototype.initialize.call(_this);
    },
    _render: function () {
      var _this = this;
      _this.$el
        .html(_this.templateFn(_this.options))
        .appendTo(document.body);
      _this.delay(16)
        .then(_this.toggle)
        .then(function () {
          return _this.delay(_this.options.delay || 2000);
        })
        .then(_this.toggle)
        .then(_this.remove);
    },
    delay: function (time) {
      if (time == null) time = this.transitionTime;
      return new Promise(function (resolve, _reject) {
        setTimeout(resolve, time);
      });
    },
    toggle: function () {
      var _this = this;
      _this.$el.toggleClass('message-show');
      return _this.delay();
    },
  });
});

define('views/Script', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var app = require('app');

  var DEFAULT_ICON = '/images/icon48.png';
  module.exports = BaseView.extend({
    className: 'script',
    attributes: {
      draggable: true,
    },
    templateUrl: '/options/templates/script.html',
    events: {
      'click [data-id=edit]': 'onEdit',
      'click [data-id=remove]': 'onRemove',
      'click [data-id=enable]': 'onEnable',
      'click [data-id=update]': 'onUpdate',
      'dragstart': 'onDragStart',
    },
    initialize: function () {
      var _this = this;
      _this.model.set('_icon', DEFAULT_ICON);
      _this.listenTo(_this.model, 'change', _this.render);
      _this.listenTo(_this.model, 'remove', _this.onRemoved);
      BaseView.prototype.initialize.call(_this);
    },
    loadIcon: function () {
      var _this = this;
      var icon = _this.model.get('meta').icon;
      if (icon && icon !== _this.model.get('_icon'))
        _this.loadImage(icon).then(function (url) {
          _this.model.set('_icon', url);
        }, function (_url) {
          _this.model.set('_icon', DEFAULT_ICON);
        });
    },
    _render: function () {
      var _this = this;
      var model = _this.model;
      var it = model.toJSON();
      it.getLocaleString = model.getLocaleString.bind(model);
      it.canUpdate = model.canUpdate();
      it.homepageURL = it.custom.homepageURL || it.meta.homepageURL || it.meta.homepage;
      it.author = _this.getAuthor(it.meta.author);
      _this.$el.html(_this.templateFn(it));
      if (!it.enabled) _this.$el.addClass('disabled');
      else _this.$el.removeClass('disabled');
      _this.loadIcon();
    },
    getAuthor: function (text) {
      if (!text) return '';
      var matches = text.match(/^(.*?)\s<(\S*?@\S*?)>$/);
      var label = _.i18n('labelAuthor');
      return matches
        ? label + '<a href=mailto:' + matches[2] + '>' + matches[1] + '</a>'
        : label + _.escape(text);
    },
    images: {},
    loadImage: function (url) {
      if (!url) return;
      var promise = this.images[url];
      if (!promise) {
        var cache = app.scriptList.cache[url];
        promise = cache ? Promise.resolve(cache)
          : new Promise(function (resolve, reject) {
            var img = new Image;
            img.onload = function () {
              resolve(url);
            };
            img.onerror = function () {
              reject(url);
            };
            img.src = url;
          });
          this.images[url] = promise;
      }
      return promise;
    },
    onEdit: function () {
      app.scriptList.trigger('edit:open', this.model);
    },
    onRemove: function () {
      var _this = this;
      _.sendMessage({
        cmd: 'RemoveScript',
        data: _this.model.id,
      }).then(function () {
        app.scriptList.remove(_this.model);
      });
    },
    onRemoved: function () {
      this.remove();
      // this.$el.remove();
    },
    onEnable: function () {
      var _this = this;
      _.sendMessage({
        cmd: 'UpdateScriptInfo',
        data: {
          id: _this.model.id,
          enabled: _this.model.get('enabled') ? 0 : 1,
        },
      });
    },
    onUpdate: function () {
      _.sendMessage({
        cmd: 'CheckUpdate',
        data: this.model.id,
      });
    },
    onDragStart: function (e) {
      var model = this.model;
      new DND(e, function (data) {
        if (data.from === data.to) return;
        _.sendMessage({
          cmd: 'Move',
          data: {
            id: model.id,
            offset: data.to - data.from,
          }
        }).then(function () {
          var collection = model.collection;
          var models = collection.models;
          var i = Math.min(data.from, data.to);
          var j = Math.max(data.from, data.to);
          var seq = [
            models.slice(0, i),
            models.slice(i, j + 1),
            models.slice(j + 1),
          ];
          i === data.to
            ? seq[1].unshift(seq[1].pop())
            : seq[1].push(seq[1].shift());
          collection.models = seq.concat.apply([], seq);
        });
      });
    },
  });

  function DND(e, cb) {
    _.bindAll(this, 'mousemove', 'mouseup');
    if (e) {
      e.preventDefault();
      this.start(e);
    }
    this.onDrop = cb;
  }
  DND.prototype.start = function (e) {
    var dragging = this.dragging = {
      el: e.currentTarget,
    };
    var $el = dragging.$el = $(dragging.el);
    var parent = $el.parent();
    var offset = $el.offset();
    dragging.offset = {
      x: e.clientX - offset.left,
      y: e.clientY - offset.top,
    };
    var next = $el.next();
    dragging.delta = (next.length ? next.offset().top : parent.height()) - offset.top;
    var children = parent.children();
    dragging.lastIndex = dragging.index = children.index($el);
    dragging.$elements = children.not($el);
    dragging.$dragged = $el.clone().addClass('dragging').css({
      left: offset.left,
      top: offset.top,
      width: offset.width,
    }).appendTo(parent);
    $el.addClass('dragging-placeholder');
    $(document).on('mousemove', this.mousemove).on('mouseup', this.mouseup);
  };
  DND.prototype.mousemove = function (e) {
    var dragging = this.dragging;
    dragging.$dragged.css({
      left: e.clientX - dragging.offset.x,
      top: e.clientY - dragging.offset.y,
    });
    var hovered = null;
    dragging.$elements.each(function (i, el) {
      var $el = $(el);
      if ($el.hasClass('dragging-moving')) return;
      var offset = $el.offset();
      var pad = 10;
      if (
        e.clientX >= offset.left + pad
          && e.clientX <= offset.left + offset.width - pad
          && e.clientY >= offset.top + pad
          && e.clientY <= offset.top + offset.height - pad
      ) {
        hovered = {
          index: i,
          el: el,
        };
        return false;
      }
    });
    if (hovered) {
      var lastIndex = dragging.lastIndex;
      var index = hovered.index;
      var isDown = index >= lastIndex;
      var $el = dragging.$el;
      var delta = dragging.delta;
      if (isDown) {
        // If moving down, the actual index should be `index + 1`
        index ++;
        $el.insertAfter(hovered.el);
      } else {
        delta = -delta;
        $el.insertBefore(hovered.el);
      }
      dragging.lastIndex = index;
      this.animate(dragging.$elements.slice(
        isDown ? lastIndex : index,
        isDown ? index : lastIndex
      ), delta);
    }
    this.checkScroll(e.clientY);
  };
  DND.prototype.animate = function ($elements, delta) {
    $elements.each(function (_i, el) {
      var $el = $(el);
      $el.addClass('dragging-moving').css({
        transition: 'none',
        transform: 'translateY(' + delta + 'px)',
      }).one('transitionend', function (e) {
        $(e.target).removeClass('dragging-moving');
      });
      setTimeout(function () {
        $el.css({
          transition: '',
          transform: '',
        });
      }, 20);
    });
  };
  DND.prototype.mouseup = function (_e) {
    $(document).off('mousemove', this.mousemove).off('mouseup', this.mouseup);
    var dragging = this.dragging;
    dragging.$dragged.remove();
    dragging.$el.removeClass('dragging-placeholder');
    this.dragging = null;
    this.onDrop && this.onDrop({
      from: dragging.index,
      to: dragging.lastIndex,
    });
  };
  DND.prototype.checkScroll = function (y) {
    var dragging = this.dragging;
    var scrollThreshold = 10;
    dragging.scroll = 0;
    var offset = dragging.el.parentNode.getBoundingClientRect();
    var delta = (y - (offset.bottom - scrollThreshold)) / scrollThreshold;
    if (delta > 0) {
      dragging.scroll = 1 + Math.min(~~ (delta * 5), 10);
    } else {
      delta = (offset.top + scrollThreshold - y) / scrollThreshold;
      if (delta > 0) dragging.scroll = -1 - Math.min(~~ (delta * 5), 10);
    }
    if (dragging.scroll) this.scrollParent();
  };
  DND.prototype.scrollParent = function () {
    function scroll() {
      var dragging = _this.dragging;
      if (dragging) {
        if (dragging.scroll) {
          dragging.el.parentNode.scrollTop += dragging.scroll;
          setTimeout(scroll, 20);
        } else dragging.scrolling = false;
      }
    }
    var _this = this;
    if (!_this.dragging.scrolling) {
      _this.dragging.scrolling = true;
      scroll();
    }
  };
});

define('views/TabAbout', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var app = _.require('app');
  module.exports = BaseView.extend({
    name: 'about',
    className: 'content',
    templateUrl: '/options/templates/tab-about.html',
    _render: function () {
      this.$el.html(this.templateFn({
        version: app.version,
      }));
    },
  });
});

define('views/TabMain', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var ScriptView = require('views/Script');
  var EditView = require('views/Edit');
  var Script = require('models').Script;
  var app = require('app');
  var tabs = _.require('utils/tabs');

  module.exports = BaseView.extend({
    name: 'main',
    className: 'content no-pad',
    templateUrl: '/options/templates/tab-installed.html',
    events: {
      'click #bNew': 'newScript',
      'click #bUpdate': 'updateAll',
      'click #bURL': 'installFromURL',
    },
    initialize: function () {
      var _this = this;
      BaseView.prototype.initialize.call(_this);
      _this.listenTo(app.scriptList, 'reset', _this.render);
      _this.listenTo(app.scriptList, 'add', _this.addOne);
      _this.listenTo(app.scriptList, 'update', _this.setBackdrop);
      _this.listenTo(app.scriptList, 'edit:open', function (model) {
        _this.closeEdit();
        _this.editView = new EditView({model: model.clone()});
        _this.$el.append(_this.editView.$el);
      });
      _this.listenTo(app.scriptList, 'edit:close', _this.closeEdit);
    },
    remove: function () {
      var _this = this;
      _this.clear();
      BaseView.prototype.remove.call(_this);
    },
    closeEdit: function () {
      var _this = this;
      if (_this.editView) {
        _this.editView.remove();
        _this.editView = null;
      }
    },
    _render: function () {
      var _this = this;
      _this.clear();
      _this.$el.html(_this.templateFn());
      _this.$list = _this.$('.scripts');
      _this.$bd = _this.$('.backdrop');
      _this.$bdm = _this.$('.backdrop > div');
      _this.setBackdrop();
      _this.addAll();
    },
    setBackdrop: function () {
      var _this = this;
      if (app.scriptList.loading) {
        _this.$bd.addClass('mask').show();
        _this.$bdm.html(_.i18n('msgLoading'));
      } else if (!app.scriptList.length) {
        _this.$bd.removeClass('mask').show();
        _this.$bdm.html(_.i18n('labelNoScripts'));
      } else {
        _this.$bd.hide();
      }
    },
    addOne: function (script) {
      var _this = this;
      var view = new ScriptView({model: script});
      _this.childViews.push(view);
      _this.$list.append(view.$el);
    },
    addAll: function () {
      app.scriptList.forEach(this.addOne, this);
    },
    newScript: function () {
      _.sendMessage({cmd: 'NewScript'}).then(function (script) {
        app.scriptList.trigger('edit:open', new Script(script));
      });
    },
    updateAll: function () {
      _.sendMessage({cmd: 'CheckUpdateAll'});
    },
    installFromURL: function () {
      var url = prompt(_.i18n('hintInputURL'));
      if (~url.indexOf('://')) {
        tabs.create({
          url: '/options.html' + '#confirm/' + encodeURIComponent(url),
        });
      }
    },
  });
});

define('views/TabSettings', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var app = require('app');
  var tabs = _.require('utils/tabs');

  var ExportList = BaseView.extend({
    templateUrl: '/options/templates/option.html',
    events: {
      'click div': 'toggleSelected',
    },
    initialize: function () {
      var _this = this;
      BaseView.prototype.initialize.call(_this);
      _this.listenTo(app.scriptList, 'reset change update', _this.render);
    },
    _render: function () {
      var _this = this;
      _this.$el.html(app.scriptList.map(function (script) {
        return _this.templateFn(script.toJSON());
      }).join(''));
    },
    getSelected: function () {
      return _.map(this.$el.children(), function (el, i) {
        return el.classList.contains('selected') ? app.scriptList.at(i) : null;
      }).filter(function (i) {return i;});
    },
    toggleSelected: function (e) {
      e.target.classList.toggle('selected');
    },
    toggleAll: function () {
      var options = this.$el.children();
      var select = _.some(options, function (option) {
        return !option.classList.contains('selected');
      });
      options.each(function (_i, option) {
        option.classList[select ? 'add' : 'remove']('selected');
      });
    },
  });

  module.exports = BaseView.extend({
    name: 'settings',
    className: 'content',
    events: {
      'change [data-check]': 'updateCheckbox',
      'change #cUpdate': 'updateAutoUpdate',
      'click #bSelect': 'toggleSelection',
      'change #bImportHelper': 'importFile',
      'click #bExport': 'exportData',
      'click #bVacuum': 'onVacuum',
    },
    templateUrl: '/options/templates/tab-settings.html',
    _render: function () {
      var _this = this;
      var options = _.options.getAll();
      _this.$el.html(_this.templateFn(options));
      _this.exportList = new ExportList({
        el: _this.$('.export-list')[0],
      });
    },
    updateCheckbox: _.updateCheckbox,
    updateAutoUpdate: function (_e) {
      _.sendMessage({cmd: 'AutoUpdate'});
    },
    toggleSelection: function () {
      this.exportList.toggleAll();
    },
    importData: function (file) {
      function getVMConfig(text) {
        var vm;
        try {
          vm = JSON.parse(text);
        } catch (e) {
          opera.postError('Error parsing ViolentMonkey configuration.');
        }
        vm = vm || {};
        _.forEach(vm.values, function (value, key) {
          _.sendMessage({
            cmd: 'SetValue',
            data: {
              uri: key,
              values: value,
            }
          });
        });
        _.forEach(vm.settings, function (value, key) {
          _.options.set(key, value);
        });
        return vm;
      }
      function getVMFile(entry, vm) {
        return entry.async('string')
          .then(function (text) {
            var script = {code: text};
            if (vm.scripts) {
              var more = vm.scripts[entry.name.slice(0, -8)];
              if (more) script.more = _.omit(more, ['id']);
            }
            return _.sendMessage({
              cmd: 'ParseScript',
              data: script,
            });
          }).then(function () {
            return true;
          });
      }
      function getVMFiles(zip) {
        var vm = zip.file('ViolentMonkey');
        return (
          vm
            ? vm.async('string')
            .then(function (text) {
              return {
                vm: getVMConfig(text),
              };
            })
            : Promise.resolve({})
        ).then(function (data) {
          data.entries = zip.filter(function (relativePath, _file) {
            return /\.user\.js$/.test(relativePath);
          });
          return data;
        });
      }
      JSZip.loadAsync(file)
        .then(getVMFiles)
        .then(function (data) {
          var vm = data.vm || {};
          return Promise.all(data.entries.map(function (entry) {
            return getVMFile(entry, vm);
          })).then(function (res) {
            return _.filter(res).length;
          });
        }).then(function (count) {
          app.scriptList.reload();
          alert(_.i18n('msgImported', [count]));
        });
    },
    importFile: function (e) {
      var _this = this;
      var input = e.target;
      if (input.files && input.files.length) {
        _this.importData(input.files[0]);
      }
    },
    exportData: function () {
      var bExport = this.$('#bExport');
      bExport.prop('disabled', true);
      var selected = this.exportList.getSelected();
      if (!selected.length) return;
      var withValues = this.$('#cbValues').prop('checked');
      _.sendMessage({
        cmd: 'ExportZip',
        data: {
          values: withValues,
          ids: _.map(selected, 'id'),
        }
      }).then(function (data) {
        var names = {};
        var vm = {
          scripts: {},
          settings: _.options.getAll(),
        };
        if (withValues) vm.values = {};
        var files = data.scripts.map(function (script) {
          var name = script.custom.name || script.meta.name || 'Noname';
          if (names[name]) name += '_' + (++ names[name]);
          else names[name] = 1;
          vm.scripts[name] = _.pick(script, ['id', 'custom', 'enabled', 'update']);
          if (withValues) {
            var values = data.values[script.uri];
            if (values) vm.values[script.uri] = values;
          }
          return {
            name: name + '.user.js',
            content: script.code,
          };
        });
        files.push({
          name: 'ViolentMonkey',
          content: JSON.stringify(vm),
        });
        return files;
      }).then(function (files) {
        var zip = new JSZip;
        files.forEach(function (file) {
          zip.file(file.name, file.content);
        });
        return zip.generateAsync({type: 'base64'});
      }).then(function (data) {
        tabs.create('data:application/zip;base64,' + data);
        bExport.prop('disabled', false);
      });
    },
    onVacuum: function (e) {
      var button = $(e.target);
      button.prop('disabled', true).html(_.i18n('buttonVacuuming'));
      _.sendMessage({cmd: 'Vacuum'}).then(function () {
        button.html(_.i18n('buttonVacuumed'));
      });
    },
  });
});
;define.use("app");