define('app', function (require, exports, _module) {
  var models = require('models');
  var Menu = models.Menu;
  var MenuItem = models.MenuItem;
  var MenuView = require('views/Menu');
  var CommandsView = require('views/Command');
  var DomainsView = require('views/Domain');
  var cache = require('cache');
  var tabs = _.require('utils/tabs');
  var badges = _.require('badges');

  var scriptsMenu = exports.scriptsMenu = new Menu;
  var commandsMenu = exports.commandsMenu = new Menu;
  var domainsMenu = exports.domainsMenu = new Menu;

  var App = cache.BaseRouter.extend({
    routes: {
      '': 'renderMenu',
      commands: 'renderCommands',
      domains: 'renderDomains',
    },
    renderMenu: function () {
      this.loadView('menu', function () {
        return new MenuView;
      });
    },
    renderCommands: function () {
      this.loadView('commands', function () {
        return new CommandsView;
      });
    },
    renderDomains: function () {
      this.loadView('domains', function () {
        return new DomainsView;
      });
    },
  });
  var app = new App('#app');
  if (!Backbone.history.start())
    app.navigate('', {trigger: true, replace: true});

  exports.navigate = app.navigate.bind(app);

  !function () {
    function commandClick(_e, model) {
      tabs.get().postMessage({
        cmd: 'Command',
        data: model.get('name'),
      });
    }
    function domainClick(_e, model) {
      tabs.create('https://greasyfork.org/scripts/search?q=' + model.get('name'));
    }
    function scriptSymbol(data) {
      return data ? 'check' : 'remove';
    }
    function scriptClick(_e, model) {
      var data = !model.get('data');
      _.sendMessage({
        cmd: 'UpdateScriptInfo',
        data: {
          id: model.get('id'),
          enabled: data,
        },
      }).then(function () {
        model.set({data: data});
      });
    }
    function init() {
      var url = tabs.get().url;
      if (/^https?:\/\//i.test(url)) {
        var matches = url.match(/:\/\/(?:www\.)?([^\/]*)/);
      var domain = matches[1];
      var pieces = domain.split('.').reverse();
      var domains = [];
      var last = pieces.shift();
      pieces.forEach(function (piece) {
        last = piece + '.' + last;
        domains.unshift(last);
      });
      if (!domains.length) domains.push(domain);
      domainsMenu.reset(domains.map(function (domain) {
        return new MenuItem({
          name: domain,
          title: true,
          className: 'ellipsis',
          onClick: domainClick,
        });
      }));
      }
    }

    var commands = {
      SetPopup: function (data, _src, _callback) {
        commandsMenu.reset(data.menus.map(function (menu) {
          return new MenuItem({
            name: menu[0],
            symbol: 'right-hand',
            title: true,
            className: 'ellipsis',
            onClick: commandClick,
          });
        }));
        _.sendMessage({
          cmd: 'GetMetas',
          data: data.ids,
        }).then(function (scripts) {
          scriptsMenu.reset(scripts.map(function (script) {
            return new MenuItem({
              id: script.id,
              name: script.custom.name || _.getLocaleString(script.meta, 'name'),
              data: !!script.enabled,
              symbol: scriptSymbol,
              title: true,
              className: 'ellipsis',
              onClick: scriptClick,
            });
          }));
        });
      },
    };
    init();
    _.bg._.popupMessenger.connect(window, commands.SetPopup);
    badges.get();
  }();
});

define('models', function (_require, exports, _module) {
  exports.MenuItem = Backbone.Model.extend({});

  exports.Menu = Backbone.Collection.extend({
    model: exports.MenuItem,
  });
});

define('views/Base', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;
  var MenuItem = require('models').MenuItem;
  var MenuItemView = require('views/MenuItem');
  var badges = _.require('badges');

  module.exports = BaseView.extend({
    templateUrl: '/popup/templates/menu.html',
    addMenuItem: function (obj, parent) {
      if (!(obj instanceof MenuItem)) obj = new MenuItem(obj);
      var item = new MenuItemView({model: obj});
      parent.append(item.$el);
      this.childViews.push(item);
    },
    components: function () {
      var $el = this.$el;
      var children = $el.children();
      return {
        top: children.first(),
        bot: children.last(),
        plh: $el.children('.placeholder'),
      };
    },
    fixStyles: function (div, plh) {
      setTimeout(function () {
        plh.html(div.html());
        var pad = div[0].offsetWidth - div[0].clientWidth + 2;
        plh.css('padding-right', pad + 'px');
        badges.button.popup.height = div.parent().height();
      });
    },
  });
});

define('views/Command', function (require, _exports, module) {
  var MenuBaseView = require('views/Base');
  var app = require('app');

  module.exports = MenuBaseView.extend({
    initialize: function () {
      MenuBaseView.prototype.initialize.call(this);
      this.listenTo(app.commandsMenu, 'reset', this.render);
    },
    _render: function () {
      if (!app.commandsMenu.length)
        return app.navigate('', {trigger: true, replace: true});
      var _this = this;
      _this.$el.html(_this.templateFn({
        hasSep: true
      }));
      var comp = _this.components();
      var top = comp.top;
      var bot = comp.bot;
      _this.addMenuItem({
        name: _.i18n('menuBack'),
        symbol: 'arrow-left',
        onClick: function (_e) {
          app.navigate('', {trigger: true});
        },
      }, top);
      app.commandsMenu.each(function (item) {
        _this.addMenuItem(item, bot);
      });
      _this.fixStyles(bot, comp.plh);
    },
  });
});

define('views/Domain', function (require, _exports, module) {
  var MenuBaseView = require('views/Base');
  var app = require('app');

  module.exports = MenuBaseView.extend({
    initialize: function () {
      MenuBaseView.prototype.initialize.call(this);
      this.listenTo(app.domainsMenu, 'reset', this.render);
    },
    _render: function () {
      var _this = this;
      _this.$el.html(_this.templateFn({
        hasSep: true
      }));
      var comp = _this.components();
      var top = comp.top;
      var bot = comp.bot;
      _this.addMenuItem({
        name: _.i18n('menuBack'),
        symbol: 'arrow-left',
        onClick: function (_e) {
          app.navigate('', {trigger: true});
        },
      }, top);
      app.domainsMenu.each(function (item) {
        _this.addMenuItem(item, bot);
      });
      _this.fixStyles(bot, comp.plh);
    },
  });
});

define('views/MenuItem', function (require, _exports, module) {
  var BaseView = require('cache').BaseView;

  module.exports = BaseView.extend({
    className: 'menu-item',
    templateUrl: '/popup/templates/menuitem.html',
    events: {
      'click .menu-item-detail': 'onClickDetail',
      'click .menu-item-label': 'onClick',
    },
    initialize: function () {
      var _this = this;
      BaseView.prototype.initialize.call(_this);
      _this.listenTo(_this.model, 'change', _this.render);
    },
    _render: function () {
      var _this = this;
      var it = _this.model.toJSON();
      if (typeof it.symbol === 'function')
        it.symbol = it.symbol(it.data);
      if (typeof it.name === 'function')
        it.name = it.name(it.data);
      _this.$el.html(_this.templateFn(it))
        .attr('title', it.title === true ? it.name : it.title);
      if (it.data === false) _this.$el.addClass('disabled');
      else _this.$el.removeClass('disabled');
      it.className && _this.$el.addClass(it.className);
      it.onClickDetail && _this.$el.addClass('has-detail');
    },
    onClick: function (e) {
      var onClick = this.model.get('onClick');
      onClick && onClick(e, this.model);
    },
    onClickDetail: function (e) {
      var onClickDetail = this.model.get('onClickDetail');
      onClickDetail && onClickDetail(e, this.model);
    },
  });
});

define('views/Menu', function (require, _exports, module) {
  var MenuBaseView = require('views/Base');
  var app = require('app');
  var badges = _.require('badges');

  module.exports = MenuBaseView.extend({
    initialize: function () {
      var _this = this;
      MenuBaseView.prototype.initialize.call(_this);
      _this.listenTo(app.scriptsMenu, 'reset', _this.render);
      _this.listenTo(app.commandsMenu, 'reset', _this.render);
      _this.listenTo(app.domainsMenu, 'reset', _this.render);
    },
    _render: function () {
      var _this = this;
      _this.$el.html(_this.templateFn({
        hasSep: !!app.scriptsMenu.length
      }));
      var comp = _this.components();
      var top = comp.top;
      var bot = comp.bot;
      _this.addMenuItem({
        name: _.i18n('menuManageScripts'),
        symbol: 'cog',
        onClick: function (_e) {
          _.bg.opera.extension.tabs.create({
            url: '/options.html',
          }).focus();
        },
      }, top);
      if (app.domainsMenu.length)
        _this.addMenuItem({
          name: _.i18n('menuFindScripts'),
          symbol: 'search',
          onClick: function (_e) {
            var tabData = badges.getData();
            var matches = tabData && tabData.url.match(/:\/\/(?:www\.)?([^\/]*)/);
            matches && _.bg.opera.extension.tabs.create({
              url: 'https://greasyfork.org/scripts/search?q=' + matches[1],
            }).focus();
          },
          onClickDetail: function (_e) {
            app.navigate('domains', {trigger: true});
          },
        }, top);
        if (app.commandsMenu.length) _this.addMenuItem({
          name: _.i18n('menuCommands'),
          symbol: 'arrow-right',
          onClick: function (_e) {
            app.navigate('commands', {trigger: true});
          },
        }, top);
        _this.addMenuItem({
          name: function (data) {
            return data ? _.i18n('menuScriptEnabled') : _.i18n('menuScriptDisabled');
          },
          data: _.options.get('isApplied'),
          symbol: function (data) {
            return data ? 'check' : 'remove';
          },
          onClick: function (_e, model) {
            var isApplied = !model.get('data');
            _.options.set('isApplied', isApplied);
            model.set({data: isApplied});
            badges.update();
          },
        }, top);
        app.scriptsMenu.each(function (item) {
          _this.addMenuItem(item, bot);
        });
        _this.fixStyles(bot, comp.plh);
    },
  });
});
;define.use("app");